package sequoia;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Set;

import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;

import maiolum.util.PropertyExpander;
import maiolum.util.URLDecoder;

import org.apache.log4j.Logger;
import org.apache.velocity.Template;
import org.apache.velocity.app.Velocity;
import org.apache.velocity.context.Context;
import org.apache.velocity.exception.VelocityException;
import org.apache.velocity.servlet.VelocityServlet;
import org.logicalcobwebs.proxool.ProxoolException;
import org.logicalcobwebs.proxool.configuration.PropertyConfigurator;

public class SequoiaServlet extends VelocityServlet {
  /**
   * Internal logger associated with this class
   */
  private static Logger logger = Logger.getLogger(SequoiaServlet.class);
  
  protected String defaultPoolId;
  
  protected List availableLanguages;
  
  protected String i18nTemplatesLocation;
  
  public static final String DEFAULT_LANGUAGE_CODE = "sk";

  public void init(ServletConfig servletConfig) throws ServletException {
    super.init(servletConfig);

    try {
      initLogging();
      initProxool();
      initAvailableLanguages();
    } catch (FileNotFoundException e) {
      throw new ServletException(e);
    }
  }
  
  protected void initLogging() {    
    String prefix = getServletContext().getRealPath(File.separator);
    String file = getInitParameter("log4j-init-file");
    // if the log4j-init-file is not set, then no point in trying
    if (file != null) {
        org.apache.log4j.PropertyConfigurator.configure(prefix + file);
        log("Logging configured to " + prefix + file);
    } else {
      log("WARN: no log4j-init-file parameter specified!");
    }
  }
  
  protected void initProxool() throws ServletException {
    try {
      String proxoolConfigLocation = getInitParameter("proxool-config-location");
      if (proxoolConfigLocation == null) {
        throw new ServletException(
            "No parameter 'proxool-config-location' was specified in the servlet config (web.xml).");
      }
      Properties expandProperties = new Properties();
      expandProperties.put("ContextLocation", getServletContext().getRealPath(
          ""));

      proxoolConfigLocation = PropertyExpander.expandParameter(
          proxoolConfigLocation, expandProperties);
      PropertyConfigurator.configure(proxoolConfigLocation);
      logger.info("Driver configured successfully");
      
      String defaultPoolId = getInitParameter("proxool-default-id");
      if (defaultPoolId == null) {
        throw new ServletException(
            "No parameter 'proxool-default-id' was specified in the servlet config (web.xml).");
      }
      synchronized (this) {
        this.defaultPoolId = defaultPoolId;
      }
    } catch (ProxoolException e) {
      throw new ServletException(e);
    }    
  }

  public Template handleRequest(HttpServletRequest request,
      HttpServletResponse response, Context context) throws Exception {

    try {
      response.setContentType("text/html; charset=windows-1250");

      String languageCode = getLanguageCode(request, DEFAULT_LANGUAGE_CODE);
      handleI18n(context, languageCode);
      
      Template template = handleSubmit(request, response, context); 
      
      return template; 
    } catch (Exception e) {
      logger.error("WARN: handleRequest failed.", e);
      return null;
    }
  }

  public Template handleSubmit(HttpServletRequest request,
      HttpServletResponse response, Context context) throws Exception {

    Template template = getTemplate("index.vm", "utf-8");

    ArrayList errors = new ArrayList();
    String query = request.getParameter("query");
    query = URLDecoder.convertFromISO(query, "windows-1250");
    
    String submitPressed = request.getParameter("submit");

    if(query == null || query.trim().length() == 0) {
      if (submitPressed != null) {
        errors.add("Dopyt mus by neprzdny!");
        context.put("errors", errors);        
      }
      return template;      
    }
    
    putQueryIntoMostRecentUsed(query, context, request);

    Connection con = null;
    Statement stmt = null;
    ResultSet rs = null;
    try {
      con = DriverManager.getConnection(this.defaultPoolId);
      stmt = con.createStatement();
      boolean queryType = stmt.execute(query);

      if (queryType) {
        ArrayList rows = new ArrayList();
        rs = stmt.getResultSet();
        int columnCount = rs.getMetaData().getColumnCount();
        int rowCount = 0;
        
        ArrayList headerCells = new ArrayList();
        for(int i = 1; i <= columnCount; i++) {
          headerCells.add(rs.getMetaData().getColumnName(i));
        }
        context.put("headerCells", headerCells);
        
        while (rs.next()) {
          ArrayList rowItems = new ArrayList();
          for (int i = 1; i <= columnCount; i++) {
            Object cellData = null;
            try {
              // try to print object as string
              cellData = rs.getString(i);
            } catch (SQLException e) {
              try {
                // .. or as Object..
                cellData = rs.getObject(i);
              } catch (SQLException ex) {
                // .. otherwise print predefined value.
                cellData = "BLOB?";
              }
            }
            rowItems.add(cellData);
          }
          rows.add(rowItems);          
          rowCount++;
        }
        context.put("affectedRows", new Integer(rowCount));
        context.put("rows", rows);
      } else {
        context.put("affectedRows", new Integer(stmt.getUpdateCount()));
      }
      context.put("queryType", new Boolean(queryType));
    } catch (SQLException e) {
      logger.error("Unable to retrieve connection", e);
      errors.add(e);
    } finally {
      try {
        if (rs != null) {
          rs.close();
        }
      } catch (SQLException e) {
        //do nothing
      }
      try {
        if (stmt != null) {
          stmt.close();
        }
      } catch (SQLException e) {
        //do nothing
      }
      try {
        if (con != null) {
          con.close();
        }
      } catch (SQLException e) {
        //do nothing
      }
    }
    context.put("errors", errors);

    return template;
  }
  
  public Context handleI18n(Context context, String languageCode) throws VelocityException {
    String path = null;
    try {
      path = this.i18nTemplatesLocation + "/" + languageCode + ".properties";
      Properties p = new Properties();
      p.load(new FileInputStream(path));

      Enumeration propertyNames = p.propertyNames();         
      while(propertyNames.hasMoreElements()) {
        String propertyKey = (String) propertyNames.nextElement();
        context.put(propertyKey, p.getProperty(propertyKey));
      }
      //put languages list into context
      context.put("languages", this.availableLanguages);
      context.put("defaultLanguage", languageCode);
      
      return context;
    } catch (FileNotFoundException e) {
      throw new VelocityException("Property file not found:" + path);
    } catch (IOException e) {
      throw new VelocityException("Cannot read property file:" + path);
    }
  }
  
  public synchronized void initAvailableLanguages() throws FileNotFoundException {
    this.availableLanguages = new ArrayList();

    String path = (String) Velocity.getProperty(Velocity.FILE_RESOURCE_LOADER_PATH);
    if(path != null) {
      File f = new File(path + "/i18n");
      if(f.exists()) {
        this.i18nTemplatesLocation = f.getPath();
        
        File[] files = f.listFiles();
        for (int i = 0; i < files.length; i++) {
          if(files[i].getName().endsWith(".properties")) {
            String fileName = files[i].getName();
            //trim .properties
            fileName = fileName.substring(0, fileName.lastIndexOf('.'));
            this.availableLanguages.add(fileName);
          }
        }            
      } else {
        throw new FileNotFoundException("Path for i18n resources does not exist: " + f);
      }
    } else {
      throw new FileNotFoundException("Unable to retrieve file resource loader path (is null).");
    }   
  }
  
  public String getLanguageCode(HttpServletRequest request, String defaultLanguageCode) {
    String languageCode;    
    String specifiedLanguage = request.getParameter("lang");
    if(specifiedLanguage != null && specifiedLanguage.trim().length() != 0) {
      log(specifiedLanguage);
      if(this.availableLanguages.contains(specifiedLanguage)) {
        return specifiedLanguage;
      }
    }
    
    return defaultLanguageCode;
  }
  
  public void putQueryIntoMostRecentUsed(String query, Context context, HttpServletRequest request) {

    HttpSession session = request.getSession();
    List previousQueries = null;
    previousQueries = (List) session.getAttribute("previousQueries");
    if(previousQueries == null) {
      previousQueries = new ArrayList();
      session.setAttribute("previousQueries", previousQueries);
    } 
    context.put("query", query);
    if(previousQueries.contains(query)) {
      Set s = new HashSet();
      s.add(query);
      previousQueries.removeAll(s);
    }
    previousQueries.add(0, query);      

    context.put("previousQueries", previousQueries);
  }
}